<?php
// $Id: taxonomy_builder.csv.inc,v 1.1 2010/03/12 19:43:51 cpliakas Exp $

/**
 * @file
 * API functions that parse CSV files into taxonomy trees.
 */

/**
 * Returns the total number of lines a file, FALSE on errors.
 *
 * @param $filename
 *   A string containing the path to the file.
 * @return
 *   An integer containing the total number of lines.
 */
function taxonomy_builder_line_count($filename) {
  if ($data = file($filename)) {
    return count($data);
  }
  return FALSE;
}

/**
 * Wrapper around fgetcsv, allows developers to get a chunk of data as opposed
 * to reading the entire file.  In addition, developers can select which columns
 * will be extracted from each line in a specific order via the $row parameter.
 * If you want to extract the 3rd, 1st, and 6th rows in that order, you would
 * set the $row parameter to the following:
 *
 * @code
 * $row = array(2, 0, 5);
 * @endcode
 *
 * @param filename
 *   A string containing the path to the file.
 * @param $line
 *   An integer containing the line to start at.  This number is NOT zero-based,
 *   meaning the first line is 1.
 * @param $limit
 *   An integer containing the maximum number of lines to parse, defaults to
 *   0 meaning no limit.
 * @param $length
 *   Must be greater than the longest line (in characters) to be found in the
 *   CSV file (allowing for trailing line-end characters).  Defaults to 4096.
 * @param $delimiter
 *   A string containing the field delimiter, defaults to ','.
 * @param $enclosure
 *   A string containing the field enclosure character, defaults to '"'.
 * @param $escape
 *   A string containing the escape character, defaults to '\\' (a backslash).
 * @return
 *   An array containing the CSV values.
 */
function taxonomy_builder_fgetcsv($filename, $line, $limit = 0, $length = 4096, $delimiter = ',', $enclosure = '"', $escape = '\\') {
  $rows = array();
  if (FALSE !== ($handle = fopen($filename, 'r'))) {

    // Skips ahead an $offset number of lines.
    for ($i = 1; $i < $line; $i++) {
      fgets($handle, $length);
    }

    // Parses data from the current pointer.
    for ($i = 0; (!$limit || $i < $limit); $i++) {
      if (FALSE !== ($row = fgetcsv($handle, $length, $delimiter, $enclosure /*, $escape */))) {
        $rows[] = $row;
      }
      else {
        break;
      }
    }

    // Closes the pointer.
    fclose($handle);
  }

  // Filters out rows with empty data, returns rows.
  return array_filter($rows, 'taxonomy_builder_filter');
}

/**
 * Filters out rows having any empty values.  Useful as an array_filter()
 * callback.
 *
 * @param $row
 *   An array containing the columns extracted from a row in a CSV file.
 * @return
 *   A boolean flagging whether the row should be stripped.
 */
function taxonomy_builder_filter($row) {
  if (!is_array($row)) {
    return FALSE;
  }
  foreach ($row as $column) {
    if (!drupal_strlen($column)) {
      return FALSE;
    }
  }
  return TRUE;
}

/**
 * Extracts columns from a row in a CSV file.
 *
 * @param $row
 *   An array containing a row of columns parsed from a CSV file.
 * @param $columns
 *   An associative array of column numbers in the order they should be
 *   extracted from the line.
 * @return
 *   An array containing the extracted rows.
 */
function taxonomy_builder_extract_columns(array $row, array $columns) {
  $row = array_intersect_key($row, drupal_map_assoc($columns));
  array_multisort($columns, $row);
  return $row;
}

/**
 * Invokes hook_taxonomy_bulder_csv_parser_arguments(), merges defaults and
 * statically caches arguments for performance.
 *
 * @param $vid
 *   An integer containing the vid of the vocabulary the terms will be added to.
 * @param $filename
 *   A string containing the path to the file being parsed.
 * @return
 *   An array of arguments.
 */
function taxonomy_builder_get_parser_arguments($vid, $filename) {
  static $arguments = array();

  // Builds the $key, checks if values are in the static.
  $key = sprintf('%s:%s', $vid, $filename);
  if (!isset($arguments[$key])) {

    // Default values for the parser.
    $default = array(
      'length' => 4096,
      'delimiter' => ',',
      'enclosure' => '"',
      'escape' => '\\',
    );

    // Gets arguments from hooks, merges defaults and strips invalid keys.
    $arguments[$key] = array_intersect_key($default, array_merge(
      $default,
      module_invoke_all('taxonomy_bulder_csv_parser_arguments', $vid, $filename)
    ));
  }

  // Returns the parser arguments.
  return $arguments[$key];
}

/**
 * Returns a batch array that simply takes values from a CSV file and imports
 * them in hierarchical order.  For example, the following CSV file will get
 * imported into a country -> state -> city hierarchy.
 *
 * @code
 * $file = <<<EOT
 * United States,Massachusetts,Boston
 * United States,Massachusetts,Worcester
 * United States,New York,New York
 * United States,New York,Albany
 * EOT;
 * @endcode
 *
 * @param $vid
 *   An integer containing the vid of the vocabulary the terms will be added to.
 * @param $filenames
 *   A array or filenames to parse, or a string containing a single file.
 * @param $limit
 *   An integer containing the maximum number of lines to parse at a time,
 *   defaults to 0 meaning no limit.
 * @return
 *   An array containing the CSV values.
 */
function taxonomy_builder_get_batch($vid, $filenames, $limit = 0) {
  // Initializes the batch array.
  $batch = array(
    'operations' => array(),
    'title' => t('Import taxonomy'),
    'init_message' => t('Importing terms ...'),
    'progress_message' => t('Batch @current out of @total'),
    'error_message' => t('An error occurred importing the terms.'),
    'file' => drupal_get_path('module', 'taxonomy_builder') .'/taxonomy_builder.csv.inc',
  );

  // Normalizes the $filenames array
  $filenames = (is_array($filenames)) ? $filenames : array($filenames);

  // Iterates over files, adds batch operations.
  foreach ($filenames as $filename) {
    if (FALSE !== ($line_count = taxonomy_builder_line_count($filename))) {
      $limit = ($limit) ? (int)$limit : $line_count;
      foreach (range(1, $line_count, $limit) as $line) {
        $batch['operations'][] = array('taxonomy_builder_batch', array(
          $vid, $filename, $line, $limit
        ));
      }
    }
  }

  // Returns the pre-populated batch array.
  return $batch;
}

/**
 * Batch function that parses lines from a CSV file into a hierarchical taxonomy
 * tree.
 *
 * @param $vid
 *   An integer containing the vid of the vocabulary the terms will be added to.
 * @param $filename
 *   A a string containing the path to the file being parsed.
 * @param $line
 *   An integer containing the line to start at.  This number is NOT zero-based,
 *   meaning the first line is 1.
 * @param $limit
 *   An integer containing the maximum number of lines to parse at a time,
 *   defaults to 0 meaning no limit.
 * @param &$context
 *   An array containing batch context information.
 * @return
 *   NULL.
 */
function taxonomy_builder_batch($vid, $filename, $line, $limit = 0, &$context) {

  // Gets parser arguments from hooks, parses a chunk of the file.
  $args = taxonomy_builder_get_parser_arguments($vid, $filename);
  $rows = taxonomy_builder_fgetcsv(
    $filename, $line, $limit, $args['length'], $args['delimiter'], $args['enclosure'], $args['escape']
  );

  // Iterates over the chunk, invokes Taxonomy Builder API hooks.
  foreach ($rows as $row) {
    $altered_row = $row;
    drupal_alter('taxonomy_builder_csv_row', $altered_row, $filename, $vid);
    $tid = taxonomy_builder_save_terms($vid, $altered_row);
    module_invoke_all('taxonomy_builder_csv_column_save', $row, $altered_row, $filename, $vid, $tid);
  }
}
